@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
-* Copyright (C) 2007, International Business Machines Corporation and    *
+* Copyright (C) 2007-2013, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 */
@@ -9,6 +11,8 @@
 
 #include "unicode/utypes.h"
 
+#if U_SHOW_CPLUSPLUS_API
+
 /**
  * \file 
  * \brief C++ API: Rule based customizable time zone
@@ -25,6 +29,12 @@ U_NAMESPACE_BEGIN
 class UVector;
 struct Transition;
 
+/**
+ * a BasicTimeZone subclass implemented in terms of InitialTimeZoneRule and TimeZoneRule instances
+ * @see BasicTimeZone
+ * @see InitialTimeZoneRule
+ * @see TimeZoneRule
+ */
 class U_I18N_API RuleBasedTimeZone : public BasicTimeZone {
 public:
     /**
@@ -34,27 +44,27 @@ public:
      * delete it.
      * @param id                The time zone ID.
      * @param initialRule       The initial time zone rule.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     RuleBasedTimeZone(const UnicodeString& id, InitialTimeZoneRule* initialRule);
 
     /**
      * Copy constructor.
      * @param source    The RuleBasedTimeZone object to be copied.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     RuleBasedTimeZone(const RuleBasedTimeZone& source);
 
     /**
      * Destructor.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual ~RuleBasedTimeZone();
 
     /**
      * Assignment operator.
      * @param right The object to be copied.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     RuleBasedTimeZone& operator=(const RuleBasedTimeZone& right);
 
@@ -64,7 +74,7 @@ public:
      * @param that  The object to be compared with.
      * @return  true if the given <code>TimeZone</code> objects are
       *semantically equal.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual UBool operator==(const TimeZone& that) const;
 
@@ -74,7 +84,7 @@ public:
      * @param that  The object to be compared with.
      * @return  true if the given <code>TimeZone</code> objects are
      * semantically unequal.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual UBool operator!=(const TimeZone& that) const;
 
@@ -91,7 +101,7 @@ public:
      * zone functions.
      * @param rule The <code>TimeZoneRule</code>.
      * @param status Output param to filled in with a success or an error.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     void addTransitionRule(TimeZoneRule* rule, UErrorCode& status);
 
@@ -102,7 +112,7 @@ public:
      * builds internal structure for making the object ready to support
      * time zone APIs such as getOffset(), getNextTransition() and others.
      * @param status Output param to filled in with a success or an error.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     void complete(UErrorCode& status);
 
@@ -111,9 +121,9 @@ public:
      * the TimeZone object cloned.
      *
      * @return   A new copy of this TimeZone object.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
-    virtual TimeZone* clone(void) const;
+    virtual RuleBasedTimeZone* clone() const;
 
     /**
      * Returns the TimeZone's adjusted GMT offset (i.e., the number of milliseconds to add
@@ -137,7 +147,7 @@ public:
      * @param millis     The reference date's milliseconds in day, local standard time
      * @param status     Output param to filled in with a success or an error.
      * @return           The offset in milliseconds to add to GMT to get local time.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                               uint8_t dayOfWeek, int32_t millis, UErrorCode& status) const;
@@ -159,7 +169,7 @@ public:
      * @param monthLength The length of the given month in days.
      * @param status     Output param to filled in with a success or an error.
      * @return           The offset in milliseconds to add to GMT to get local time.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual int32_t getOffset(uint8_t era, int32_t year, int32_t month, int32_t day,
                            uint8_t dayOfWeek, int32_t millis,
@@ -185,7 +195,7 @@ public:
      * effect, this value is zero; otherwise it is a positive value,
      * typically one hour.
      * @param ec input-output error code
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual void getOffset(UDate date, UBool local, int32_t& rawOffset,
                            int32_t& dstOffset, UErrorCode& ec) const;
@@ -195,7 +205,7 @@ public:
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @param offsetMillis  The new raw GMT offset for this time zone.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual void setRawOffset(int32_t offsetMillis);
 
@@ -204,7 +214,7 @@ public:
      * to GMT to get local time, before taking daylight savings time into account).
      *
      * @return   The TimeZone's raw GMT offset.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual int32_t getRawOffset(void) const;
 
@@ -212,10 +222,11 @@ public:
      * Queries if this time zone uses daylight savings time.
      * @return true if this time zone uses daylight savings time,
      * false, otherwise.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual UBool useDaylightTime(void) const;
 
+#ifndef U_FORCE_HIDE_DEPRECATED_API
     /**
      * Queries if the given date is in daylight savings time in
      * this time zone.
@@ -230,6 +241,7 @@ public:
      * @deprecated ICU 2.4. Use Calendar::inDaylightTime() instead.
      */
     virtual UBool inDaylightTime(UDate date, UErrorCode& status) const;
+#endif  // U_FORCE_HIDE_DEPRECATED_API
 
     /**
      * Returns true if this zone has the same rule and offset as another zone.
@@ -237,7 +249,7 @@ public:
      * @param other the <code>TimeZone</code> object to be compared with
      * @return true if the given zone is the same as this one,
      * with the possible exception of the ID
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual UBool hasSameRules(const TimeZone& other) const;
 
@@ -247,9 +259,9 @@ public:
      * @param inclusive Whether the base time is inclusive or not.
      * @param result    Receives the first transition after the base time.
      * @return  TRUE if the transition is found.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
-    virtual UBool getNextTransition(UDate base, UBool inclusive, TimeZoneTransition& result) /*const*/;
+    virtual UBool getNextTransition(UDate base, UBool inclusive, TimeZoneTransition& result) const;
 
     /**
      * Gets the most recent time zone transition before the base time.
@@ -257,9 +269,9 @@ public:
      * @param inclusive Whether the base time is inclusive or not.
      * @param result    Receives the most recent transition before the base time.
      * @return  TRUE if the transition is found.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
-    virtual UBool getPreviousTransition(UDate base, UBool inclusive, TimeZoneTransition& result) /*const*/;
+    virtual UBool getPreviousTransition(UDate base, UBool inclusive, TimeZoneTransition& result) const;
 
     /**
      * Returns the number of <code>TimeZoneRule</code>s which represents time transitions,
@@ -267,9 +279,9 @@ public:
      * <code>InitialTimeZoneRule</code>.  The return value range is 0 or any positive value.
      * @param status    Receives error status code.
      * @return The number of <code>TimeZoneRule</code>s representing time transitions.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
-    virtual int32_t countTransitionRules(UErrorCode& status) /*const*/;
+    virtual int32_t countTransitionRules(UErrorCode& status) const;
 
     /**
      * Gets the <code>InitialTimeZoneRule</code> and the set of <code>TimeZoneRule</code>
@@ -285,17 +297,17 @@ public:
      *                      the timezone transition rules.  On output, actual number of
      *                      rules filled in the array will be set.
      * @param status        Receives error status code.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual void getTimeZoneRules(const InitialTimeZoneRule*& initial,
-        const TimeZoneRule* trsrules[], int32_t& trscount, UErrorCode& status) /*const*/;
+        const TimeZoneRule* trsrules[], int32_t& trscount, UErrorCode& status) const;
 
     /**
      * Get time zone offsets from local wall time.
      * @internal
      */
     virtual void getOffsetFromLocal(UDate date, int32_t nonExistingTimeOpt, int32_t duplicatedTimeOpt,
-        int32_t& rawOffset, int32_t& dstOffset, UErrorCode& status) /*const*/;
+        int32_t& rawOffset, int32_t& dstOffset, UErrorCode& status) const;
 
 private:
     void deleteRules(void);
@@ -311,6 +323,7 @@ private:
         int32_t NonExistingTimeOpt, int32_t DuplicatedTimeOpt) const;
     void getOffsetInternal(UDate date, UBool local, int32_t NonExistingTimeOpt, int32_t DuplicatedTimeOpt,
         int32_t& rawOffset, int32_t& dstOffset, UErrorCode& ec) const;
+    void completeConst(UErrorCode &status) const;
 
     InitialTimeZoneRule *fInitialRule;
     UVector             *fHistoricRules;
@@ -328,7 +341,7 @@ public:
      * .       erived::getStaticClassID()) ...
      * </pre>
      * @return          The class ID for all objects of this class.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     static UClassID U_EXPORT2 getStaticClassID(void);
 
@@ -341,7 +354,7 @@ public:
      * @return          The class ID for this object. All objects of a
      *                  given class have the same class ID.  Objects of
      *                  other classes have different class IDs.
-     * @draft ICU 3.8
+     * @stable ICU 3.8
      */
     virtual UClassID getDynamicClassID(void) const;
 };
@@ -350,6 +363,8 @@ U_NAMESPACE_END
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
 
+#endif /* U_SHOW_CPLUSPLUS_API */
+
 #endif // RBTZ_H
 
 //eof
