@@ -1,12 +1,17 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 1997-2001, International Business Machines Corporation and
+ * Copyright (c) 1997-2009, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 
 #ifndef _INTLTESTDATEFORMATAPI
 #define _INTLTESTDATEFORMATAPI
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
 
 #include "intltest.h"
 
@@ -33,6 +38,13 @@ private:
      * Test that no parse or format methods are hidden.
      */
     void TestNameHiding(void);
+
+    /**
+     * Add better code coverage.
+     */
+    void TestCoverage(void);
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif
