@@ -59,22 +59,8 @@
 #include "dcfmtest.h"       // DecimalFormatTest
 #include "listformattertest.h"  // ListFormatterTest
 #include "regiontst.h"      // RegionTest
-
-// NumberFormatter is disabled on some platforms due to C++11 compatibility
-#if !UPRV_INCOMPLETE_CPP11_SUPPORT
-#   include "numbertest.h"     // All NumberFormatter tests
-#else
-class NumberTest : public IntlTest {
-  public:
-    void runIndexedTest(int32_t index, UBool exec, const char*& name, char*) {
-        if (index > 0) { name = ""; return; } // base case
-        name = "NumberTest";
-        if (exec) {
-            infoln(u"   NOTE: NumberTest is disabled on this platform; see ICU ticket #13393.");
-        }
-    }
-};
-#endif
+#include "numbertest.h"     // NumberTest
+#include "erarulestest.h"   // EraRulesTest
 
 extern IntlTest *createCompactDecimalFormatTest();
 extern IntlTest *createGenderInfoTest();
@@ -85,7 +71,9 @@ extern IntlTest *createTimeUnitTest();
 extern IntlTest *createMeasureFormatTest();
 extern IntlTest *createNumberFormatSpecificationTest();
 extern IntlTest *createScientificNumberFormatterTest();
-extern IntlTest *createNumberFormat2Test(); 
+extern IntlTest *createFormattedValueTest();
+extern IntlTest *createFormattedStringBuilderTest();
+extern IntlTest *createStringSegmentTest();
 
 
 #define TESTCLASS(id, TestClass)          \
@@ -229,16 +217,36 @@ void IntlTestFormat::runIndexedTest( int32_t index, UBool exec, const char* &nam
             callTest(*test, par);
           }
           break;
-        case 50:
-          name = "NumberFormat2Test"; 
-          if (exec) { 
-            logln("NumberFormat2Test test---"); 
-            logln((UnicodeString)""); 
-            LocalPointer<IntlTest> test(createNumberFormat2Test()); 
-            callTest(*test, par); 
-          } 
-          break;
+        TESTCLASS(50,NumberFormatDataDrivenTest);
         TESTCLASS(51,NumberTest);
+        TESTCLASS(52,EraRulesTest);
+        case 53:
+          name = "FormattedValueTest";
+          if (exec) {
+            logln("FormattedValueTest test---");
+            logln((UnicodeString)"");
+            LocalPointer<IntlTest> test(createFormattedValueTest());
+            callTest(*test, par);
+          }
+          break;
+        case 54:
+          name = "FormattedStringBuilderTest";
+          if (exec) {
+            logln("FormattedStringBuilderTest test---");
+            logln((UnicodeString)"");
+            LocalPointer<IntlTest> test(createFormattedStringBuilderTest());
+            callTest(*test, par);
+          }
+          break;
+        case 55:
+          name = "StringSegmentTest";
+          if (exec) {
+            logln("StringSegmentTest test---");
+            logln((UnicodeString)"");
+            LocalPointer<IntlTest> test(createStringSegmentTest());
+            callTest(*test, par);
+          }
+          break;
         default: name = ""; break; //needed to end loop
     }
     if (exec) {
