@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /****************************************************************************
  * COPYRIGHT: 
- * Copyright (c) 2001-2005, International Business Machines Corporation and others
+ * Copyright (c) 2001-2008, International Business Machines Corporation and others
  * All Rights Reserved.
  ***************************************************************************/
 
@@ -9,7 +11,7 @@
 
 #include "unicode/utypes.h"
 
-#if !UCONFIG_NO_COLLATION
+#if !UCONFIG_NO_COLLATION 
 
 #include "unicode/tblcoll.h"
 #include "unicode/brkiter.h"
@@ -27,7 +29,7 @@ public:
 
     void runIndexedTest(int32_t index, UBool exec, const char* &name, 
                         char* par = NULL);
-
+#if !UCONFIG_NO_BREAK_ITERATION
 private:
     RuleBasedCollator *m_en_us_; 
     RuleBasedCollator *m_fr_fr_;
@@ -88,7 +90,8 @@ private:
     void TestUClassID();
     void TestSubclass();
     void TestCoverage();
-    void TestDiactricMatch();
+    void TestDiacriticMatch();
+#endif
 };
 
 #endif /* #if !UCONFIG_NO_COLLATION */
