@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
- * Copyright (c) 2001, International Business Machines Corporation and
+ * Copyright (c) 2001-2005, International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
 /************************************************************************
@@ -37,11 +39,15 @@ public:
     
     void TestUnicodeSetErrors(void);
 
-    void TestUniToHexErrors(void);
+    //void TestUniToHexErrors(void);
 
     void TestRBTErrors(void);
 
-    void TestHexToUniErrors(void);
+    //void TestHexToUniErrors(void);
+
+    // JitterBug 4452, for coverage.  The reason to put this method here is 
+    //  this class is comparable smaller than other Transliterator*Test classes
+    void TestCoverage(void);
 
 };
 
