@@ -1,12 +1,14 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
 *
-*   Copyright (C) 1999-2003, International Business Machines
+*   Copyright (C) 1999-2016 International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
 *   file name:  gencnval.c
-*   encoding:   US-ASCII
+*   encoding:   UTF-8
 *   tab size:   8 (not used)
 *   indentation:4
 *
@@ -28,6 +30,7 @@
 #include "ucnv_io.h"
 #include "cmemory.h"
 #include "cstring.h"
+#include "uinvchar.h"
 #include "filestrm.h"
 #include "unicode/uclean.h"
 #include "unewdata.h"
@@ -83,7 +86,7 @@ static const UDataInfo dataInfo={
     0,
 
     {0x43, 0x76, 0x41, 0x6c},     /* dataFormat="CvAl" */
-    {3, 0, 0, 0},                 /* formatVersion */
+    {3, 0, 1, 0},                 /* formatVersion */
     {1, 4, 2, 0}                  /* dataVersion */
 };
 
@@ -134,7 +137,19 @@ static uint16_t aliasListsSize = 0;
 /* Were the standard tags declared before the aliases. */
 static UBool standardTagsUsed = FALSE;
 static UBool verbose = FALSE;
-static int32_t lineNum = 1;
+static UBool quiet = FALSE;
+static int lineNum = 1;
+
+static UConverterAliasOptions tableOptions = {
+    UCNV_IO_STD_NORMALIZED,
+    1 /* containsCnvOptionInfo */
+};
+
+
+/**
+ * path to convrtrs.txt
+ */
+const char *path;
 
 /* prototypes --------------------------------------------------------------- */
 
@@ -191,7 +206,8 @@ enum
     VERBOSE,
     COPYRIGHT,
     DESTDIR,
-    SOURCEDIR
+    SOURCEDIR,
+    QUIET
 };
 
 static UOption options[]={
@@ -200,13 +216,14 @@ static UOption options[]={
     UOPTION_VERBOSE,
     UOPTION_COPYRIGHT,
     UOPTION_DESTDIR,
-    UOPTION_SOURCEDIR
+    UOPTION_SOURCEDIR,
+    UOPTION_QUIET
 };
 
 extern int
 main(int argc, char* argv[]) {
+    int i, n;
     char pathBuf[512];
-    const char *path;
     FileStream *in;
     UNewDataMemory *out;
     UErrorCode errorCode=U_ZERO_ERROR;
@@ -215,7 +232,7 @@ main(int argc, char* argv[]) {
 
     /* preset then read command line options */
     options[DESTDIR].value=options[SOURCEDIR].value=u_getDataDirectory();
-    argc=u_parseArgs(argc, argv, sizeof(options)/sizeof(options[0]), options);
+    argc=u_parseArgs(argc, argv, UPRV_LENGTHOF(options), options);
 
     /* error handling, printing usage message */
     if(argc<0) {
@@ -230,6 +247,7 @@ main(int argc, char* argv[]) {
             "options:\n"
             "\t-h or -? or --help  this usage text\n"
             "\t-v or --verbose     prints out extra information about the alias table\n"
+            "\t-q or --quiet       do not display warnings and progress\n"
             "\t-c or --copyright   include a copyright notice\n"
             "\t-d or --destdir     destination directory, followed by the path\n"
             "\t-s or --sourcedir   source directory, followed by the path\n",
@@ -241,23 +259,26 @@ main(int argc, char* argv[]) {
         verbose = TRUE;
     }
 
-    if(argc>=2) {
-        path=argv[1];
-    } else {
-        path=options[SOURCEDIR].value;
-        if(path!=NULL && *path!=0) {
-            char *end;
+    if(options[QUIET].doesOccur) {
+        quiet = TRUE;
+    }
 
-            uprv_strcpy(pathBuf, path);
-            end = uprv_strchr(pathBuf, 0);
-            if(*(end-1)!=U_FILE_SEP_CHAR) {
-                *(end++)=U_FILE_SEP_CHAR;
-            }
-            uprv_strcpy(end, "convrtrs.txt");
-            path=pathBuf;
-        } else {
-            path = "convrtrs.txt";
+    if (argc >= 2) {
+        path = argv[1];
+    } else {
+        path = "convrtrs.txt";
+    }
+
+    const char* sourcedir = options[SOURCEDIR].value;
+    if (sourcedir != NULL && *sourcedir != 0) {
+        char *end;
+        uprv_strcpy(pathBuf, sourcedir);
+        end = uprv_strchr(pathBuf, 0);
+        if(*(end-1)!=U_FILE_SEP_CHAR) {
+            *(end++)=U_FILE_SEP_CHAR;
         }
+        uprv_strcpy(end, path);
+        path = pathBuf;
     }
 
     uprv_memset(stringStore, 0, sizeof(stringStore));
@@ -270,14 +291,14 @@ main(int argc, char* argv[]) {
 
     in=T_FileStream_open(path, "r");
     if(in==NULL) {
-        fprintf(stderr, "gencnval: unable to open input file convrtrs.txt\n");
+        fprintf(stderr, "gencnval: unable to open input file %s\n", path);
         exit(U_FILE_ACCESS_ERROR);
     }
     parseFile(in);
     T_FileStream_close(in);
 
     /* create the output file */
-    out=udata_create(options[DESTDIR].value, DATA_TYPE, U_ICUDATA_NAME "_" DATA_NAME, &dataInfo,
+    out=udata_create(options[DESTDIR].value, DATA_TYPE, DATA_NAME, &dataInfo,
                      options[COPYRIGHT].doesOccur ? U_COPYRIGHT_STRING : NULL, &errorCode);
     if(U_FAILURE(errorCode)) {
         fprintf(stderr, "gencnval: unable to open output file - error %s\n", u_errorName(errorCode));
@@ -294,6 +315,15 @@ main(int argc, char* argv[]) {
         exit(errorCode);
     }
 
+    /* clean up tags */
+    for (i = 0; i < MAX_TAG_COUNT; i++) {
+        for (n = 0; n < MAX_CONV_COUNT; n++) {
+            if (tags[i].aliasList[n].aliases!=NULL) {
+                uprv_free(tags[i].aliasList[n].aliases);
+            }
+        }
+    }
+
     return 0;
 }
 
@@ -319,7 +349,7 @@ parseFile(FileStream *in) {
         /* Read non-empty lines that don't start with a space character. */
         while (T_FileStream_readLine(in, lastLine, MAX_LINE_SIZE) != NULL) {
             lastLineSize = chomp(lastLine);
-            if (lineSize == 0 || (lastLineSize > 0 && isspace(*lastLine))) {
+            if (lineSize == 0 || (lastLineSize > 0 && isspace((int)*lastLine))) {
                 uprv_strcpy(line + lineSize, lastLine);
                 lineSize += lastLineSize;
             } else if (lineSize > 0) {
@@ -330,12 +360,12 @@ parseFile(FileStream *in) {
         }
 
         if (validParse || lineSize > 0) {
-            if (isspace(*line)) {
-                fprintf(stderr, "error(line %d): cannot start an alias with a space\n", lineNum-1);
+            if (isspace((int)*line)) {
+                fprintf(stderr, "%s:%d: error: cannot start an alias with a space\n", path, lineNum-1);
                 exit(U_PARSE_ERROR);
             } else if (line[0] == '{') {
                 if (!standardTagsUsed && line[lineSize - 1] != '}') {
-                    fprintf(stderr, "error(line %d): alias needs to start with a converter name\n", lineNum);
+                    fprintf(stderr, "%s:%d: error: alias needs to start with a converter name\n", path, lineNum);
                     exit(U_PARSE_ERROR);
                 }
                 addOfficialTaggedStandards(line, lineSize);
@@ -345,7 +375,7 @@ parseFile(FileStream *in) {
                     parseLine(line);
                 }
                 else {
-                    fprintf(stderr, "error(line %d): alias table needs to start a list of standard tags\n", lineNum);
+                    fprintf(stderr, "%s:%d: error: alias table needs to start a list of standard tags\n", path, lineNum);
                     exit(U_PARSE_ERROR);
                 }
             }
@@ -375,7 +405,7 @@ chomp(char *line) {
             *s = 0;
             break;
         }
-        if (!isspace(*s)) {
+        if (!isspace((int)*s)) {
             lastNonSpace = s;
         }
         ++s;
@@ -406,7 +436,7 @@ parseLine(const char *line) {
 
     /* get the converter name */
     start=pos;
-    while(line[pos]!=0 && !isspace(line[pos])) {
+    while(line[pos]!=0 && !isspace((int)line[pos])) {
         ++pos;
     }
     limit=pos;
@@ -425,7 +455,7 @@ parseLine(const char *line) {
     for(;;) {
 
         /* skip white space */
-        while(line[pos]!=0 && isspace(line[pos])) {
+        while(line[pos]!=0 && isspace((int)line[pos])) {
             ++pos;
         }
 
@@ -436,7 +466,7 @@ parseLine(const char *line) {
 
         /* get an alias name */
         start=pos;
-        while(line[pos]!=0 && line[pos]!='{' && !isspace(line[pos])) {
+        while(line[pos]!=0 && line[pos]!='{' && !isspace((int)line[pos])) {
             ++pos;
         }
         limit=pos;
@@ -458,7 +488,7 @@ parseLine(const char *line) {
         /* addAlias(alias, 0, cnv, FALSE);*/
 
         /* skip whitespace */
-        while (line[pos] && isspace(line[pos])) {
+        while (line[pos] && isspace((int)line[pos])) {
             ++pos;
         }
 
@@ -467,7 +497,7 @@ parseLine(const char *line) {
             ++pos;
             do {
                 start = pos;
-                while (line[pos] && line[pos] != '}' && !isspace( line[pos])) {
+                while (line[pos] && line[pos] != '}' && !isspace((int)line[pos])) {
                     ++pos;
                 }
                 limit = pos;
@@ -478,7 +508,7 @@ parseLine(const char *line) {
                     addAlias(alias, tag, cnv, (UBool)(line[limit-1] == '*'));
                 }
 
-                while (line[pos] && isspace(line[pos])) {
+                while (line[pos] && isspace((int)line[pos])) {
                     ++pos;
                 }
             } while (line[pos] && line[pos] != '}');
@@ -486,7 +516,7 @@ parseLine(const char *line) {
             if (line[pos] == '}') {
                 ++pos;
             } else {
-                fprintf(stderr, "error(line %d): Unterminated tag list\n", lineNum);
+                fprintf(stderr, "%s:%d: Unterminated tag list\n", path, lineNum);
                 exit(U_UNMATCHED_BRACES);
             }
         } else {
@@ -502,7 +532,7 @@ getTagNumber(const char *tag, uint16_t tagLen) {
     UBool preferredName = ((tagLen > 0) ? (tag[tagLen - 1] == '*') : (FALSE));
 
     if (tagCount >= MAX_TAG_COUNT) {
-        fprintf(stderr, "error(line %d): too many tags\n", lineNum);
+        fprintf(stderr, "%s:%d: too many tags\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
 
@@ -520,7 +550,7 @@ getTagNumber(const char *tag, uint16_t tagLen) {
 
     /* we need to add this tag */
     if (tagCount >= MAX_TAG_COUNT) {
-        fprintf(stderr, "error(line %d): too many tags\n", lineNum);
+        fprintf(stderr, "%s:%d: error: too many tags\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
 
@@ -528,13 +558,13 @@ getTagNumber(const char *tag, uint16_t tagLen) {
     atag = allocString(&tagBlock, tag, tagLen);
 
     if (standardTagsUsed) {
-        fprintf(stderr, "error(line %d): Tag \"%s\" is not declared at the beginning of the alias table.\n",
-            lineNum, atag);
+        fprintf(stderr, "%s:%d: error: Tag \"%s\" is not declared at the beginning of the alias table.\n",
+            path, lineNum, atag);
         exit(1);
     }
     else if (tagLen > 0 && strcmp(tag, ALL_TAG_STR) != 0) {
-        fprintf(stderr, "warning(line %d): Tag \"%s\" was added to the list of standards because it was not declared at beginning of the alias table.\n",
-            lineNum, atag);
+        fprintf(stderr, "%s:%d: warning: Tag \"%s\" was added to the list of standards because it was not declared at beginning of the alias table.\n",
+            path, lineNum, atag);
     }
 
     /* add the tag to the tag table */
@@ -552,15 +582,29 @@ addTaggedAlias(uint16_t tag, const char *alias, uint16_t converter) {
 
 static void
 addOfficialTaggedStandards(char *line, int32_t lineLen) {
+    (void) lineLen; // suppress compiler warnings about unused variable
     char *atag;
-    char *tag = strchr(line, '{') + 1;
+    char *endTagExp;
+    char *tag;
     static const char WHITESPACE[] = " \t";
 
     if (tagCount > UCNV_NUM_RESERVED_TAGS) {
-        fprintf(stderr, "error(line %d): official tags already added\n", lineNum);
+        fprintf(stderr, "%s:%d: error: official tags already added\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
-    strchr(tag, '}')[0] = 0;
+    tag = strchr(line, '{');
+    if (tag == NULL) {
+        /* Why were we called? */
+        fprintf(stderr, "%s:%d: error: Missing start of tag group\n", path, lineNum);
+        exit(U_PARSE_ERROR);
+    }
+    tag++;
+    endTagExp = strchr(tag, '}');
+    if (endTagExp == NULL) {
+        fprintf(stderr, "%s:%d: error: Missing end of tag group\n", path, lineNum);
+        exit(U_PARSE_ERROR);
+    }
+    endTagExp[0] = 0;
 
     tag = strtok(tag, WHITESPACE);
     while (tag != NULL) {
@@ -588,7 +632,7 @@ addToKnownAliases(const char *alias) {
         if (knownAliases[idx] != num
             && uprv_strcmp(alias, GET_ALIAS_STR(knownAliases[idx])) == 0)
         {
-            fprintf(stderr, "warning(line %d): duplicate alias %s and %s found\n",
+            fprintf(stderr, "%s:%d: warning: duplicate alias %s and %s found\n", path, 
                 lineNum, alias, GET_ALIAS_STR(knownAliases[idx]));
             duplicateKnownAliasesCount++;
             break;
@@ -597,7 +641,7 @@ addToKnownAliases(const char *alias) {
             && ucnv_compareNames(alias, GET_ALIAS_STR(knownAliases[idx])) == 0)
         {
             if (verbose) {
-                fprintf(stderr, "information(line %d): duplicate alias %s and %s found\n",
+                fprintf(stderr, "%s:%d: information: duplicate alias %s and %s found\n", path, 
                     lineNum, alias, GET_ALIAS_STR(knownAliases[idx]));
             }
             duplicateKnownAliasesCount++;
@@ -606,8 +650,8 @@ addToKnownAliases(const char *alias) {
     }
 */
     if (knownAliasesCount >= MAX_ALIAS_COUNT) {
-        fprintf(stderr, "warning(line %d): Too many aliases defined for all converters\n",
-            lineNum);
+        fprintf(stderr, "%s:%d: warning: Too many aliases defined for all converters\n",
+            path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
     /* TODO: We could try to unlist exact duplicates. */
@@ -620,27 +664,26 @@ addToKnownAliases(const char *alias) {
 static uint16_t
 addAlias(const char *alias, uint16_t standard, uint16_t converter, UBool defaultName) {
     uint32_t idx, idx2;
-    UBool dupFound = FALSE;
     UBool startEmptyWithoutDefault = FALSE;
     AliasList *aliasList;
 
     if(standard>=MAX_TAG_COUNT) {
-        fprintf(stderr, "error(line %d): too many standard tags\n", lineNum);
+        fprintf(stderr, "%s:%d: error: too many standard tags\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
     if(converter>=MAX_CONV_COUNT) {
-        fprintf(stderr, "error(line %d): too many converter names\n", lineNum);
+        fprintf(stderr, "%s:%d: error: too many converter names\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
     aliasList = &tags[standard].aliasList[converter];
 
     if (strchr(alias, '}')) {
-        fprintf(stderr, "error(line %d): unmatched } found\n",
+        fprintf(stderr, "%s:%d: error: unmatched } found\n", path, 
             lineNum);
     }
 
     if(aliasList->aliasCount + 1 >= MAX_TC_ALIAS_COUNT) {
-        fprintf(stderr, "error(line %d): too many aliases for alias %s and converter %s\n",
+        fprintf(stderr, "%s:%d: error: too many aliases for alias %s and converter %s\n", path, 
             lineNum, alias, GET_ALIAS_STR(converters[converter].converter));
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
@@ -662,19 +705,7 @@ addAlias(const char *alias, uint16_t standard, uint16_t converter, UBool default
         }
     }
 
-    /* Check for duplicates in a tag/converter combination */
-    for (idx = 0; idx < aliasList->aliasCount; idx++) {
-        uint16_t aliasNum = tags[standard].aliasList[converter].aliases[idx];
-        if (aliasNum && ucnv_compareNames(alias, GET_ALIAS_STR(aliasNum)) == 0 && standard != ALL_TAG_NUM)
-        {
-            fprintf(stderr, "warning(line %d): duplicate alias %s and %s found for standard %s\n",
-                lineNum, alias, GET_ALIAS_STR(aliasNum), GET_TAG_STR(tags[standard].tag));
-            dupFound = TRUE;
-            break;
-        }
-    }
-
-    if (!dupFound && standard != ALL_TAG_NUM) {
+    if (standard != ALL_TAG_NUM) {
         /* Check for duplicate aliases for this tag on all converters */
         for (idx = 0; idx < converterCount; idx++) {
             for (idx2 = 0; idx2 < tags[standard].aliasList[idx].aliasCount; idx2++) {
@@ -682,9 +713,25 @@ addAlias(const char *alias, uint16_t standard, uint16_t converter, UBool default
                 if (aliasNum
                     && ucnv_compareNames(alias, GET_ALIAS_STR(aliasNum)) == 0)
                 {
-                    fprintf(stderr, "warning(line %d): duplicate alias %s found for standard tag %s between converter %s and converter %s\n",
-                        lineNum, alias, GET_TAG_STR(tags[standard].tag), GET_ALIAS_STR(converters[converter].converter), GET_ALIAS_STR(converters[idx].converter));
-                    dupFound = TRUE;
+                    if (idx == converter) {
+                        /*
+                         * (alias, standard) duplicates are harmless if they map to the same converter.
+                         * Only print a warning in verbose mode, or if the alias is a precise duplicate,
+                         * not just a lenient-match duplicate.
+                         */
+                        if (verbose || 0 == uprv_strcmp(alias, GET_ALIAS_STR(aliasNum))) {
+                            fprintf(stderr, "%s:%d: warning: duplicate aliases %s and %s found for standard %s and converter %s\n", path, 
+                                lineNum, alias, GET_ALIAS_STR(aliasNum),
+                                GET_TAG_STR(tags[standard].tag),
+                                GET_ALIAS_STR(converters[converter].converter));
+                        }
+                    } else {
+                        fprintf(stderr, "%s:%d: warning: duplicate aliases %s and %s found for standard tag %s between converter %s and converter %s\n", path, 
+                            lineNum, alias, GET_ALIAS_STR(aliasNum),
+                            GET_TAG_STR(tags[standard].tag),
+                            GET_ALIAS_STR(converters[converter].converter),
+                            GET_ALIAS_STR(converters[idx].converter));
+                    }
                     break;
                 }
             }
@@ -699,7 +746,7 @@ addAlias(const char *alias, uint16_t standard, uint16_t converter, UBool default
                     if (aliasNum
                         && ucnv_compareNames(alias, GET_ALIAS_STR(aliasNum)) == 0)
                     {
-                        fprintf(stderr, "warning(line %d): duplicate alias %s found for converter %s and standard tag %s\n",
+                        fprintf(stderr, "%s:%d: warning: duplicate alias %s found for converter %s and standard tag %s\n", path, 
                             lineNum, alias, GET_ALIAS_STR(converters[converter].converter), GET_TAG_STR(tags[standard].tag));
                         break;
                     }
@@ -718,7 +765,7 @@ addAlias(const char *alias, uint16_t standard, uint16_t converter, UBool default
     }
     if (defaultName) {
         if (aliasList->aliases[0] != 0) {
-            fprintf(stderr, "error(line %d): Alias %s and %s cannot both be the default alias for standard tag %s and converter %s\n",
+            fprintf(stderr, "%s:%d: error: Alias %s and %s cannot both be the default alias for standard tag %s and converter %s\n", path, 
                 lineNum,
                 alias,
                 GET_ALIAS_STR(aliasList->aliases[0]),
@@ -742,13 +789,13 @@ static uint16_t
 addConverter(const char *converter) {
     uint32_t idx;
     if(converterCount>=MAX_CONV_COUNT) {
-        fprintf(stderr, "error(line %d): too many converters\n", lineNum);
+        fprintf(stderr, "%s:%d: error: too many converters\n", path, lineNum);
         exit(U_BUFFER_OVERFLOW_ERROR);
     }
 
     for (idx = 0; idx < converterCount; idx++) {
         if (ucnv_compareNames(converter, GET_ALIAS_STR(converters[idx].converter)) == 0) {
-            fprintf(stderr, "error(line %d): duplicate converter %s found!\n", lineNum, converter);
+            fprintf(stderr, "%s:%d: error: duplicate converter %s found!\n", path, lineNum, converter);
             exit(U_PARSE_ERROR);
             break;
         }
@@ -792,7 +839,8 @@ resolveAliasToConverter(uint16_t alias, uint16_t *tagNum, uint16_t *converterNum
     }
     *tagNum = UINT16_MAX;
     *converterNum = UINT16_MAX;
-    fprintf(stderr, "warning: alias %s not found\n",
+    fprintf(stderr, "%s: warning: alias %s not found\n",
+        path,
         GET_ALIAS_STR(alias));
     return;
 }
@@ -806,59 +854,64 @@ resolveAliases(uint16_t *uniqueAliasArr, uint16_t *uniqueAliasToConverterArr, ui
     uint16_t currConvNum, oldConvNum;
     const char *lastName;
 
-    resolveAliasToConverter(knownAliases[0], &oldTagNum, &currConvNum);
-    uniqueAliasToConverterArr[uniqueAliasIdx] = currConvNum;
-    oldConvNum = currConvNum;
-    uniqueAliasArr[uniqueAliasIdx] = knownAliases[0] + aliasOffset;
-    uniqueAliasIdx++;
-    lastName = GET_ALIAS_STR(knownAliases[0]);
+    if (knownAliasesCount != 0) {
+      resolveAliasToConverter(knownAliases[0], &oldTagNum, &currConvNum);
+      uniqueAliasToConverterArr[uniqueAliasIdx] = currConvNum;
+      oldConvNum = currConvNum;
+      uniqueAliasArr[uniqueAliasIdx] = knownAliases[0] + aliasOffset;
+      uniqueAliasIdx++;
+      lastName = GET_ALIAS_STR(knownAliases[0]);
 
-    for (idx = 1; idx < knownAliasesCount; idx++) {
-        resolveAliasToConverter(knownAliases[idx], &currTagNum, &currConvNum);
-        if (ucnv_compareNames(lastName, GET_ALIAS_STR(knownAliases[idx])) == 0) {
-            /* duplicate found */
-            if ((currTagNum < oldTagNum && currTagNum >= UCNV_NUM_RESERVED_TAGS)
-                || oldTagNum == 0) {
-                oldTagNum = currTagNum;
-                uniqueAliasToConverterArr[uniqueAliasIdx - 1] = currConvNum;
-                uniqueAliasArr[uniqueAliasIdx - 1] = knownAliases[idx] + aliasOffset;
-                if (verbose) {
-                    printf("using %s instead of %s -> %s", 
-                        GET_ALIAS_STR(knownAliases[idx]),
-                        lastName,
-                        GET_ALIAS_STR(converters[currConvNum].converter));
-                    if (oldConvNum != currConvNum) {
-                        printf(" (alias conflict)");
-                    }
-                    puts("");
-                }
-            }
-            else {
-                /* else ignore it */
-                if (verbose) {
-                    printf("folding %s into %s -> %s",
-                        GET_ALIAS_STR(knownAliases[idx]),
-                        lastName,
-                        GET_ALIAS_STR(converters[oldConvNum].converter));
-                    if (oldConvNum != currConvNum) {
-                        printf(" (alias conflict)");
-                    }
-                    puts("");
-                }
-            }
-            if (oldConvNum != currConvNum) {
-                uniqueAliasToConverterArr[uniqueAliasIdx - 1] |= UCNV_AMBIGUOUS_ALIAS_MAP_BIT;
-            }
-        }
-        else {
-            uniqueAliasToConverterArr[uniqueAliasIdx] = currConvNum;
-            oldConvNum = currConvNum;
-            uniqueAliasArr[uniqueAliasIdx] = knownAliases[idx] + aliasOffset;
-            uniqueAliasIdx++;
-            lastName = GET_ALIAS_STR(knownAliases[idx]);
-            oldTagNum = currTagNum;
-            /*printf("%s -> %s\n", GET_ALIAS_STR(knownAliases[idx]), GET_ALIAS_STR(converters[currConvNum].converter));*/
-        }
+      for (idx = 1; idx < knownAliasesCount; idx++) {
+          resolveAliasToConverter(knownAliases[idx], &currTagNum, &currConvNum);
+          if (ucnv_compareNames(lastName, GET_ALIAS_STR(knownAliases[idx])) == 0) {
+              /* duplicate found */
+              if ((currTagNum < oldTagNum && currTagNum >= UCNV_NUM_RESERVED_TAGS)
+                  || oldTagNum == 0) {
+                  oldTagNum = currTagNum;
+                  uniqueAliasToConverterArr[uniqueAliasIdx - 1] = currConvNum;
+                  uniqueAliasArr[uniqueAliasIdx - 1] = knownAliases[idx] + aliasOffset;
+                  if (verbose) {
+                      printf("using %s instead of %s -> %s",
+                          GET_ALIAS_STR(knownAliases[idx]),
+                          lastName,
+                          GET_ALIAS_STR(converters[currConvNum].converter));
+                      if (oldConvNum != currConvNum) {
+                          printf(" (alias conflict)");
+                      }
+                      puts("");
+                  }
+              }
+              else {
+                  /* else ignore it */
+                  if (verbose) {
+                      printf("folding %s into %s -> %s",
+                          GET_ALIAS_STR(knownAliases[idx]),
+                          lastName,
+                          GET_ALIAS_STR(converters[oldConvNum].converter));
+                      if (oldConvNum != currConvNum) {
+                          printf(" (alias conflict)");
+                      }
+                      puts("");
+                  }
+              }
+              if (oldConvNum != currConvNum) {
+                  uniqueAliasToConverterArr[uniqueAliasIdx - 1] |= UCNV_AMBIGUOUS_ALIAS_MAP_BIT;
+              }
+          }
+          else {
+              uniqueAliasToConverterArr[uniqueAliasIdx] = currConvNum;
+              oldConvNum = currConvNum;
+              uniqueAliasArr[uniqueAliasIdx] = knownAliases[idx] + aliasOffset;
+              uniqueAliasIdx++;
+              lastName = GET_ALIAS_STR(knownAliases[idx]);
+              oldTagNum = currTagNum;
+              /*printf("%s -> %s\n", GET_ALIAS_STR(knownAliases[idx]), GET_ALIAS_STR(converters[currConvNum].converter));*/
+          }
+          if (uprv_strchr(GET_ALIAS_STR(converters[currConvNum].converter), UCNV_OPTION_SEP_CHAR) != NULL) {
+              uniqueAliasToConverterArr[uniqueAliasIdx-1] |= UCNV_CONTAINS_OPTION_BIT;
+          }
+      }
     }
     return uniqueAliasIdx;
 }
@@ -888,15 +941,16 @@ createOneAliasList(uint16_t *aliasArrLists, uint32_t tag, uint32_t converter, ui
                 value = aliasList->aliases[aliasNum] + offset;
             } else {
                 value = 0;
-                if (tag != 0) { /* Only show the warning when it's not the leftover tag. */
-                    printf("warning: tag %s does not have a default alias for %s\n",
+                if (tag != 0 && !quiet) { /* Only show the warning when it's not the leftover tag. */
+                    fprintf(stderr, "%s: warning: tag %s does not have a default alias for %s\n",
+                            path,
                             GET_TAG_STR(tags[tag].tag),
                             GET_ALIAS_STR(converters[converter].converter));
                 }
             }
             aliasLists[aliasListsSize++] = value;
             if (aliasListsSize >= MAX_LIST_SIZE) {
-                fprintf(stderr, "error: Too many alias lists\n");
+                fprintf(stderr, "%s: error: Too many alias lists\n", path);
                 exit(U_BUFFER_OVERFLOW_ERROR);
             }
 
@@ -904,6 +958,26 @@ createOneAliasList(uint16_t *aliasArrLists, uint32_t tag, uint32_t converter, ui
     }
 }
 
+static void
+createNormalizedAliasStrings(char *normalizedStrings, const char *origStringBlock, int32_t stringBlockLength) {
+    int32_t currStrLen;
+    uprv_memcpy(normalizedStrings, origStringBlock, stringBlockLength);
+    while ((currStrLen = (int32_t)uprv_strlen(origStringBlock)) < stringBlockLength) {
+        int32_t currStrSize = currStrLen + 1;
+        if (currStrLen > 0) {
+            int32_t normStrLen;
+            ucnv_io_stripForCompare(normalizedStrings, origStringBlock);
+            normStrLen = (int32_t)uprv_strlen(normalizedStrings);
+            if (normStrLen > 0) {
+                uprv_memset(normalizedStrings + normStrLen, 0, currStrSize - normStrLen);
+            }
+        }
+        stringBlockLength -= currStrSize;
+        normalizedStrings += currStrSize;
+        origStringBlock += currStrSize;
+    }
+}
+
 static void
 writeAliasTable(UNewDataMemory *out) {
     uint32_t i, j;
@@ -927,7 +1001,12 @@ writeAliasTable(UNewDataMemory *out) {
     }
 
     /* Write the size of the TOC */
-    udata_write32(out, 8);
+    if (tableOptions.stringNormalizationType == UCNV_IO_UNNORMALIZED) {
+        udata_write32(out, 8);
+    }
+    else {
+        udata_write32(out, 9);
+    }
 
     /* Write the sizes of each section */
     /* All sizes are the number of uint16_t units, not bytes */
@@ -937,8 +1016,11 @@ writeAliasTable(UNewDataMemory *out) {
     udata_write32(out, uniqueAliasesSize);  /* The preresolved form of mapping an untagged the alias to a converter */
     udata_write32(out, tagCount * converterCount);
     udata_write32(out, aliasListsSize + 1);
-    udata_write32(out, 0);  /* Reserved space. */
+    udata_write32(out, sizeof(tableOptions) / sizeof(uint16_t));
     udata_write32(out, (tagBlock.top + stringBlock.top) / sizeof(uint16_t));
+    if (tableOptions.stringNormalizationType != UCNV_IO_UNNORMALIZED) {
+        udata_write32(out, (tagBlock.top + stringBlock.top) / sizeof(uint16_t));
+    }
 
     /* write the table of converters */
     /* Think of this as the column headers */
@@ -972,14 +1054,29 @@ writeAliasTable(UNewDataMemory *out) {
     /* Write the lists */
     udata_writeBlock(out, (const void *)aliasLists, aliasListsSize * sizeof(uint16_t));
 
+    /* Write any options for the alias table. */
+    udata_writeBlock(out, (const void *)&tableOptions, sizeof(tableOptions));
+
     /* write the tags strings */
     udata_writeString(out, tagBlock.store, tagBlock.top);
 
     /* write the aliases strings */
     udata_writeString(out, stringBlock.store, stringBlock.top);
 
-    uprv_free(aliasArrLists);
+    /* write the normalized aliases strings */
+    if (tableOptions.stringNormalizationType != UCNV_IO_UNNORMALIZED) {
+        char *normalizedStrings = (char *)uprv_malloc(tagBlock.top + stringBlock.top);
+        createNormalizedAliasStrings(normalizedStrings, tagBlock.store, tagBlock.top);
+        createNormalizedAliasStrings(normalizedStrings + tagBlock.top, stringBlock.store, stringBlock.top);
+
+        /* Write out the complete normalized array. */
+        udata_writeString(out, normalizedStrings, tagBlock.top + stringBlock.top);
+        uprv_free(normalizedStrings);
+    }
+
+    uprv_free(uniqueAliasesToConverter);
     uprv_free(uniqueAliases);
+    uprv_free(aliasArrLists);
 }
 
 static char *
@@ -999,7 +1096,7 @@ allocString(StringBlock *block, const char *s, int32_t length) {
     top=block->top + (uint32_t)((length + 1 + 1) & ~1);
 
     if(top >= block->max) {
-        fprintf(stderr, "error(line %d): out of memory\n", lineNum);
+        fprintf(stderr, "%s:%d: error: out of memory\n", path, lineNum);
         exit(U_MEMORY_ALLOCATION_ERROR);
     }
 
@@ -1013,7 +1110,7 @@ allocString(StringBlock *block, const char *s, int32_t length) {
 
     /* check for invariant characters now that we have a NUL-terminated string for easy output */
     if(!uprv_isInvariantString(p, length)) {
-        fprintf(stderr, "error(line %d): the name %s contains not just invariant characters\n", lineNum, p);
+        fprintf(stderr, "%s:%d: error: the name %s contains not just invariant characters\n", path, lineNum, p);
         exit(U_INVALID_TABLE_FORMAT);
     }
 
